﻿// TexturePackerImporter 1.03

using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;
using UnityEditor.Sprites;
using System;
using System.IO;

[Serializable]
public class FrameData
{
    public int x;
    public int y;
    public int w;
    public int h;
}

[Serializable]
public class SpriteSourceSize
{
    public int x;
    public int y;
    public int w;
    public int h;
}

[Serializable]
public class SourceSize
{
    public int w;
    public int h;
}

[Serializable]
public class Pivot
{
    public float x;
    public float y;
}

[Serializable]
public class Frame
{
    public string filename;
    public FrameData frame;
    public bool rotated;
    public bool trimmed;
    public SpriteSourceSize spriteSourceSize;
    public SourceSize sourceSize;
    public Pivot pivot;
}

[Serializable]
public class Frames
{
    public Frame[] frame;
}

[Serializable]
public class ImageSize
{
    public int w;
    public int h;
}

[Serializable]
public class Meta
{
    public string app;
    public string version;
    public string image;
    public string format;
    public ImageSize size;
}

[Serializable]
public class TextureAtlasJson
{
    public Frame[] frames;
    public Meta meta;
}

public class TextureAtlasImporter : AssetPostprocessor
{
    static void OnPostprocessAllAssets(string[] imported_assets, string[] deleted_assets, string[] moved_assets, string[] moved_from_asset_paths)
    {
        foreach(var asset in imported_assets)
        {
            if(!Path.GetExtension(asset).Equals(".json"))
            {
                continue;
            }
            TextAsset json = AssetDatabase.LoadAssetAtPath(asset, typeof(TextAsset)) as TextAsset;
            string png_path = asset.Substring(0, asset.Length - 5) + ".png";
            if(!File.Exists(png_path))
            {
                continue;
            }
            //Debug.Log("png_path = " + png_path);
            //Debug.Log("json.text = " + json.text);
            TextureAtlas(png_path, json.text);
        }
    }

    static void TextureAtlas(string png_path, string json_text)
    {
        TextureAtlasJson texture_atlas_json = JsonUtility.FromJson<TextureAtlasJson>(json_text);
        if(texture_atlas_json == null)
        {
            return;
        }
        if(texture_atlas_json.meta == null)
        {
            return;
        }
        if(texture_atlas_json.meta.app == null)
        {
            return;
        }
        if((!texture_atlas_json.meta.app.Contains("texturepacker")) && (!texture_atlas_json.meta.app.Contains("TextureAtlas")))
        {
            return;
        }
        TextureImporter texture_importer = (TextureImporter)TextureImporter.GetAtPath(png_path);
        texture_importer.textureType = TextureImporterType.Sprite;
        float height = (float)texture_atlas_json.meta.size.h;
        List<SpriteMetaData> sprite_meta_data_list = new List<SpriteMetaData>();
        for(int i = 0; i < texture_atlas_json.frames.Length; ++ i)
        {
            SpriteMetaData sprite_meta_data = new SpriteMetaData();
            string filename = texture_atlas_json.frames[i].filename.Substring(0, texture_atlas_json.frames[i].filename.Length - 4);
//          Debug.Log("filename = " + filename);
            sprite_meta_data.name = filename;
            // 座標とサイズ
            float x = (float)texture_atlas_json.frames[i].frame.x;
            float y = (float)texture_atlas_json.frames[i].frame.y;
            float w = (float)texture_atlas_json.frames[i].frame.w;
            float h = (float)texture_atlas_json.frames[i].frame.h;
            sprite_meta_data.rect = new Rect(x, height - (y + h), w, h);
            // pivot計算
            float pivot_x = 0.5f;
            float pivot_y = 0.5f;
            if(texture_atlas_json.frames[i].pivot == null)
            {
                sprite_meta_data.alignment = 0; // Center
            }
            else
            {
                sprite_meta_data.alignment = 9; // Custom
                pivot_x = texture_atlas_json.frames[i].pivot.x;
                pivot_y = texture_atlas_json.frames[i].pivot.y;
            }
            // 余白がある場合はpivotを修正する
            float left_margin = (float)texture_atlas_json.frames[i].spriteSourceSize.x;
            float top_margin = (float)texture_atlas_json.frames[i].spriteSourceSize.y;
            float right_margin = (float)texture_atlas_json.frames[i].sourceSize.w - w - left_margin;
            float bottom_margin = (float)texture_atlas_json.frames[i].sourceSize.h - h - top_margin;
            if((left_margin > 0.0f) || (top_margin > 0.0f) || (right_margin > 0.0f) || (bottom_margin > 0.0f))
            {
                sprite_meta_data.alignment = 9; // Custom
                pivot_x = ((float)texture_atlas_json.frames[i].sourceSize.w * pivot_x - left_margin) / w;
                pivot_y = ((float)texture_atlas_json.frames[i].sourceSize.h * pivot_y - top_margin) / h;
            }
            sprite_meta_data.pivot = new Vector2(pivot_x, 1.0f - pivot_y);
            sprite_meta_data_list.Add(sprite_meta_data);
//          Debug.Log("sprite_meta_data.name = " + sprite_meta_data.name);
        }
        // import
        texture_importer.spriteImportMode = SpriteImportMode.Multiple;
        texture_importer.spritesheet = sprite_meta_data_list.ToArray();
        // appry
        EditorUtility.SetDirty(texture_importer);
        AssetDatabase.ImportAsset(png_path, ImportAssetOptions.ForceUncompressedImport);
    }
}
